@@ -1,26 +1,32 @@
 ## File Name: immer_jml.R
-## File Version: 0.9636
+## File Version: 1.011
 
 
-immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL, irtmodel="PCM",
+immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL,
+                weights=NULL, irtmodel="PCM",
                 pid=NULL, rater=NULL, eps=.3, est_method="eps_adj",
-                maxiter=1000, conv=1E-5, max_incr=3, maxiter_update=10, maxiter_line_search=6,
-                conv_update=1E-5,
-                verbose=TRUE, use_Rcpp=TRUE, shortcut=TRUE )
+                maxiter=1000, conv=1E-5, max_incr=3, incr_fac=1.1, maxiter_update=10, maxiter_line_search=6,
+                conv_update=1E-5, verbose=TRUE, use_Rcpp=TRUE, shortcut=TRUE )
 {
     time <- list( start=Sys.time() )
     CALL <- match.call()
 
     #-- process rating data
-    res <- immer_proc_data( dat=dat, pid=pid, rater=rater, weights=NULL, maxK=maxK)
+    res <- immer_proc_data( dat=dat, pid=pid, rater=rater, weights=weights, maxK=maxK)
     dat <- res$dat2.NA
     pid <- res$pid
     maxK <- res$maxK
     K <- res$K
     pseudoitems_design <- res$pseudoitems_design
 
+    use_weights <- FALSE
+    if (! is.null(weights)){
+        use_Rcpp <- TRUE
+        use_weights <- TRUE
+    }
+
     #-- shortcut for analyzing the dataset
-    res <- immer_jml_proc_shortcut( dat=dat, pid=pid, shortcut=shortcut, weights=NULL)
+    res <- immer_jml_proc_shortcut( dat=dat, pid=pid, shortcut=shortcut, weights=weights)
     dat <- res$dat
     pid <- res$pid
     shortcut <- res$shortcut
@@ -71,7 +77,6 @@ immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL, i
                                 max_pers=person$max_pers, eps=eps)
     }
 
-
     dat_score <- array( dat_resp * person$eps_i, dim=c(N,I,K+1) )
     dat_score2 <- dat_score
     for (ii in 1:I){
@@ -144,11 +149,12 @@ immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL, i
         xsi0 <- xsi
         theta0 <- theta
         deviance0 <- deviance
-
         #** update item parameters
-        args_item <- list( score_items=score_items, I=I, K=K, b=b, A=A, xsi=xsi, theta=theta, N=N, dat_resp=dat_resp,
-                    max_incr=max_incr, maxiter_update=maxiter_update, conv_update=conv_update,
-                    b_fixed=b_fixed, ItemScore=ItemScore, shortcut_index=shortcut_index, weights=weights )
+        args_item <- list( score_items=score_items, I=I, K=K, b=b, A=A, xsi=xsi,
+                    theta=theta, N=N, dat_resp=dat_resp, max_incr=max_incr,
+                    maxiter_update=maxiter_update, conv_update=conv_update,
+                    b_fixed=b_fixed, ItemScore=ItemScore, shortcut_index=shortcut_index,
+                    weights=weights, use_weights=use_weights )
         res <- do.call( what=fct_item, args=args_item)
         b <- res$b
         xsi <- res$xsi
@@ -164,6 +170,11 @@ immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL, i
         theta_der2 <- res$theta_der2
         probs <- res$probs
 
+        #* trim theta increment
+        incr <- theta - theta0
+        max_incr <- max_incr/incr_fac
+        theta <- theta0 + immer_trim_increment(incr=incr, max_incr=max_incr)
+
         #** calculate log-likelihood
         loglike <- immer_jml_calc_loglike( dat_score=dat_score, probs=probs, K=K, weights=weights)
         deviance <- -2*loglike
@@ -174,11 +185,12 @@ immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL, i
 
         #--- line search in case of non-decreasing deviance
         if (deviance > deviance0 ){
-            immer_jml_print_progress_line_search( verbose=verbose, deviance=deviance, digits_deviance=6)
+            # immer_jml_print_progress_line_search( verbose=verbose, deviance=deviance, digits_deviance=6)
             lambda <- 1
             lambda_fac <- 2
-            # lambda_fac <- 1.4
             xsi_old <- xsi0
+            b_old <- b
+            probs_old <- probs
             xsi_new <- xsi
             theta_old <- theta0
             theta_new <- theta
@@ -203,9 +215,14 @@ immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL, i
                 loglike <- immer_jml_calc_loglike( dat_score=dat_score, probs=probs, K=K, weights=weights)
                 deviance <- -2*loglike
                 iter_in <- iter_in + 1
-                immer_jml_print_progress_line_search( verbose=verbose, deviance=deviance, digits_deviance=6)
+                # immer_jml_print_progress_line_search( verbose=verbose, deviance=deviance, digits_deviance=6)
             }
-        }
+            if (iter_in >=maxiter_ls){
+                xsi <- .5*(xsi_old + xsi)
+                theta <- .5*(theta_old + theta)
+                b <- immer_jml_calc_item_intercepts(A=A, xsi=xsi)
+            }
+        }  #-- end line search
 
         deviance.history[iter+1] <- deviance
 
@@ -224,7 +241,8 @@ immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL, i
         if ( (item_parm_change < conv) & (pers_parm_change < conv )){
             iterate <- FALSE
         }
-
+        deviance_change <- abs(deviance0 - deviance) / abs(deviance)
+        if (deviance_change < 1E-12){ iterate <- FALSE }
         if (verbose){
             v2 <- paste0( " | Deviance change=", round( deviance0 - deviance, 6) )
             if ( iter==1){ v2 <- "" }
@@ -260,7 +278,7 @@ immer_jml <- function(dat, A=NULL, maxK=NULL, center_theta=TRUE, b_fixed=NULL, i
         probs <- res$probs
     }
 
-    #-- standard errors
+    #-- coompute standard errors
     eps0 <- 1E-20
     # se_adj <- bc_adj_fac
     se_adj <- 1
